# -*- coding: utf-8 -*-

import datetime

import requests

from foobot.error import *

__all__ = [
    'FoobotAPI',
    'FoobotDataItem',
]


def check_execption(func):
    def _check(*arg, **kws):
        resp = func(*arg, **kws)
        if resp.status_code >= 400:
            if resp.status_code == 403:
                raise FoobotOAuthError(401, 'UNAUTHORIZED')
            else:
                raise FoobotAPIError(resp)
        body = resp.content
        if body:
            return resp.json()
        return body

    return _check


class APIBase(object):
    @property
    def headers(self):
        return {
            'Authorization': 'bearer {}'.format(
                self.credentials['access_token']),
            'X-API-KEY-TOKEN': self.credentials['api_key']
        }

    @check_execption
    def _get(self, url, base_url=None, **opts):
        if 'headers' not in opts:
            opts['headers'] = self.headers
        return requests.get(self._url(url, base_url=base_url), **opts)

    @check_execption
    def _post(self, url, **opts):
        return requests.post(self._url(url), **opts)

    def _url(self, url, base_url=None):
        return "https://{}/{}".format(
            self.credentials['base_url'] if base_url is None else base_url,
            url
        )


class FoobotDataItem(object):
    timestamp = None
    particulate_matter = None
    temp = None
    humidity = None
    co2 = None
    voc = None
    foobot_index = None

    def __init__(self, data):
        self.timestamp = datetime.datetime.fromtimestamp(int(data[0]))
        self.particulate_matter = data[1]
        self.temp = data[2]
        self.humidity = data[3]
        self.co2 = data[4]
        self.voc = data[5]
        self.foobot_index = data[6]


class FoobotAPI(APIBase):
    def __init__(self,
                 api_key,
                 homehost,
                 username,
                 access_token,
                 client_key,
                 client_secret):
        self.base_url = 'api-us-east-1.foobot.io'
        self.home_host = homehost
        self.username = username

        self.api_key = api_key
        self.access_token = access_token
        self.client_key = client_key
        self.client_secret = client_secret

    @property
    def credentials(self):
        return {
            'client_key': self.client_key,
            'client_secret': self.client_secret,
            'access_token': self.access_token,
            'api_key': self.api_key,
            'home_host': self.home_host,
            'base_url': self.base_url,
            'username': self.username,
        }

    @property
    def device(self):
        return self._get(
            'v2/owner/{}/device/'.format(self.credentials['username']),
            base_url=self.credentials['home_host'],
        )

    def get_devices(self):
        return self.device

    @property
    def region_token(self):
        return self._get('v2/user/me/home/',
                         base_url=self.credentials['home_host'])

    def refresh_token(self, refresh_token):
        data = {
            'client_id': self.credentials['client_key'],
            'client_secret': self.credentials['client_secret'],
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token,
        }
        return self._post('oauth2/token', data=data)

    def get_data(self, start=None, end='last', average_by=3600):
        if start is None:
            start = 3600 * 24  # pragma: no cover
        data = []
        for dev in self.get_devices():
            url = 'v2/device/{}/datapoint/{}/{}/{}/'.format(
                dev['uuid'],
                start,
                end,
                average_by
            )
            item = self._get(url, base_url=self.credentials['home_host'])
            if item:
                data.append(item)
        return data

    def get_last_day_data(self, raw=False):
        data = self.get_data(start=3600 * 24, end='last')

        if raw:
            return data

        for i in range(len(data)):
            data[i]['datapoints'] = [FoobotDataItem(d) for d in
                                     data[i]['datapoints']]
        return data


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import unittest
import datetime

from foobot.api import (FoobotAPI, FoobotDataItem)
from unittest.mock import patch


class FoobotDataItemTestCase(unittest.TestCase):
    def test_create_data_item(self):
        now = datetime.datetime.now()
        data = [now.strftime("%s"), '1', '2', '3', '4', '5', '6']
        data_item = FoobotDataItem(data)
        self.assertTrue(data_item.timestamp.year == now.year)
        self.assertTrue(data_item.timestamp.month == now.month)
        self.assertTrue(data_item.timestamp.day == now.day)
        self.assertTrue(data_item.timestamp.hour == now.hour)
        self.assertTrue(data_item.timestamp.minute == now.minute)
        self.assertTrue(data_item.timestamp.second == now.second)
        self.assertTrue(data_item.particulate_matter == data[1])
        self.assertTrue(data_item.temp == data[2])
        self.assertTrue(data_item.humidity == data[3])
        self.assertTrue(data_item.co2 == data[4])
        self.assertTrue(data_item.voc == data[5])
        self.assertTrue(data_item.foobot_index == data[6])


class FoobotTestCase(unittest.TestCase):
    def create_test_api(self):
        return FoobotAPI(
            api_key='TEST_API_KEY',
            access_token='TEST_API_TOKEN',
            username='TEST_USERNAME',
            homehost='api-eu-west-1.foobot.io',
            client_key='',
            client_secret='',
        )

    def test_create_api(self):
        api = self.create_test_api()
        self.assertTrue(api.access_token == 'TEST_API_TOKEN')
        self.assertTrue(api.username == 'TEST_USERNAME')
        self.assertTrue(api.home_host == 'http://api.foobot.io/')

    def test_get_devices(self):
        api = self.create_test_api()
        with patch('foobot.api.requests.get') as mock_devices:
            mock_devices.return_value.ok = True
            mock_devices.return_value.status_code = 200
            mock_devices.return_value.json.return_value = [{
                'mac': '123',
                'userId': 0,
                'name': 'MyFoobot',
                'uuid': '123'
            }]
            data = api.get_devices()
            self.assertTrue(data is not None)
            self.assertTrue(data != {})
            self.assertTrue(data[0]['userId'] == 0)
            self.assertTrue(data[0]['uuid'] == '123')
            self.assertTrue(data[0]['name'] == 'MyFoobot')
            self.assertTrue(data[0]['mac'] == '123')

    def test_bad_get_devices(self):
        api = self.create_test_api()
        with patch('foobot.api.requests.get') as mock_devices:
            mock_devices.return_value.ok = True
            mock_devices.return_value.status_code = 404
            mock_devices.return_value.json.return_value = [{
                'mac': '123',
                'userId': 0,
                'name': 'MyFoobot',
                'uuid': '123'
            }]
            data = api.get_devices()
            self.assertTrue(data == {})

    def test_get_last_day_data_raw(self):
        api = self.create_test_api()
        with patch('foobot.api.requests.get') as mock_get, \
            patch.object(FoobotAPI, 'get_devices') as mock_get_devices:
            mocked_json = {
                'datapoints': [
                    ['28372837', '1', '2', '3', '4', '5', '6'],
                    ['28372838', '11', '12', '13', '14', '15', '16'],
                    ['28372838', '21', '22', '23', '24', '25', '26'],
                ]
            }
            mock_get_devices.return_value = [{
                'mac': '123',
                'userId': 0,
                'name': 'MyFoobot',
                'uuid': '123'
            }]

            mock_get.return_value.ok = True
            mock_get.return_value.status_code = 200
            mock_get.return_value.json.return_value = mocked_json

            data = api.get_last_day_data(raw=True)
            self.assertTrue(data is not None)
            self.assertTrue(data != {})
            self.assertTrue('datapoints' in data[0])
            for i, d in enumerate(data[0]['datapoints']):
                self.assertTrue(mocked_json['datapoints'][i] == d)

    def test_get_last_day_data_items(self):
        api = self.create_test_api()
        with patch('foobot.api.requests.get') as mock_get, \
            patch.object(FoobotAPI, 'get_devices') as mock_get_devices:
            mocked_json = {
                'datapoints': [
                    ['28372837', '1', '2', '3', '4', '5', '6'],
                    ['28372838', '11', '12', '13', '14', '15', '16'],
                    ['28372838', '21', '22', '23', '24', '25', '26'],
                ]
            }
            mock_get_devices.return_value = [{
                'mac': '123',
                'userId': 0,
                'name': 'MyFoobot',
                'uuid': '123'
            }]

            rt_json = mocked_json.copy()

            mock_get.return_value.ok = True
            mock_get.return_value.status_code = 200
            mock_get.return_value.json.return_value = mocked_json

            data = api.get_last_day_data(raw=False)
            self.assertTrue(data is not None)
            self.assertTrue(data != {})
            self.assertTrue('datapoints' in data[0])
            for i, d in enumerate(rt_json['datapoints']):
                self.assertTrue(FoobotDataItem(d).particulate_matter == data[0]['datapoints'][i].particulate_matter)
                self.assertTrue(FoobotDataItem(d).temp == data[0]['datapoints'][i].temp)
                self.assertTrue(FoobotDataItem(d).humidity == data[0]['datapoints'][i].humidity)
                self.assertTrue(FoobotDataItem(d).co2 == data[0]['datapoints'][i].co2)
                self.assertTrue(FoobotDataItem(d).voc == data[0]['datapoints'][i].voc)
                self.assertTrue(FoobotDataItem(d).foobot_index == data[0]['datapoints'][i].foobot_index)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

__all__ = [
    'FoobotBaseError',
    'FoobotOAuthError',
    'FoobotAPIError',
    'FoobotRefreshTokenError',
]


class FoobotBaseError(Exception):
    def __str__(self):
        return "*** %s (%s) *** : %s" % (self.status, self.reason, self.msg)


class FoobotOAuthError(FoobotBaseError):
    def __init__(self, status, reason, msg={}):
        self.status = status
        self.reason = reason
        self.msg = {}


class FoobotAPIError(FoobotBaseError):
    def __init__(self, resp):
        self.status = resp.status_code
        self.reason = resp.reason
        self.msg = resp.text


class FoobotRefreshTokenError(FoobotBaseError):
    pass



from setuptools import setup, find_packages

PACKAGE = "foobot"
NAME = "foobot"
DESCRIPTION = "python API wrapper for foobot.io"
AUTHOR = "Welltory"
AUTHOR_EMAIL = "hello@welltory.com"
URL = "https://github.com/Welltory/python-foobot"
VERSION = __import__(PACKAGE).__version__


requirements = [i.strip() for i in open("requirements.txt").readlines()]

test_requirements = [i.strip() for i in
                     open("requirements_test.txt").readlines()]


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=open("README.rst").read(),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="BSD",
    url=URL,
    packages=find_packages(exclude=["tests.*", "tests"]),
    test_suite='tests',
    tests_require=test_requirements,
    install_requires=requirements,
    keywords=['air quality', 'sensor', 'IoT'],
    classifiers=[
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
    ],
    zip_safe=False,
)